
#ifndef softmin_hh
#define softmin_hh

#include <algorithm>
#include <list.hh>
#include <math.h>

static const double beta = 1;

template <typename T>
inline
T softmin(T t)
{
  return t;
}

template <typename Itr>
inline
typename std::iterator_traits<Itr>::value_type softmin(Itr begin, Itr end)
{
  typename std::iterator_traits<Itr>::value_type n;
  if (begin == end) {
    empty(n);
    return n;
  }
  assert(!isEmpty(*begin));
  
  Itr tmp_begin = begin;
  //identify the strict minimum. This allows for better numerical accuracy
  typename std::iterator_traits<Itr>::value_type strict_min = *tmp_begin;
  int minIdx = 0;
  int i = 0;
  for(; tmp_begin != end; ++tmp_begin) {
  	if(*tmp_begin < strict_min){
  		strict_min = *tmp_begin;
  		minIdx = i;
  	}
  	i++;
  }
  assert(begin != end);
  typename std::iterator_traits<Itr>::value_type normalization = 0;
  n = 0;
  typename std::iterator_traits<Itr>::value_type weight;
  i = 0;
  for (; begin != end; ++begin) {
    if(i == minIdx){
    	normalization += 1;
    	n += *begin;
    } else {
  		weight = exp(-((*begin) - strict_min ) * beta);
  		normalization += weight;
  		n += weight * (*begin);
  	}
  	i++;
  }
  return n / normalization;
}

template <typename Iterator>
inline
typename std::iterator_traits<Iterator>::value_type
softmin(std::pair<Iterator, Iterator> &p)
{
  return softmin(p.first, p.second);
}

#endif
