
#ifndef softmin_deriv_hh
#define softmin_deriv_hh

#include <algorithm>
#include <list.hh>
#include <math.h>
#include "soft_derivative.hh"
#include "softmin.hh"

template <typename T>
inline
T softmin_deriv(T t)
{
  return t;
}

template <typename Itr>
inline
soft_derivative softmin_deriv(Itr begin, Itr end)
{
	soft_derivative n;
	n.deriv = (*begin).deriv;
	if (begin == end) {
		empty(n);
		return n;
	}
	assert(!isEmpty(*begin));
  
	for(unsigned int i = 0; i < (*begin).deriv.size() ; i++){
	  	for(unsigned int j = 0; j < (*begin).deriv.size() ; j++){
	  		n.deriv.set(i,j,0);
	 	}
	}
  
  Itr tmp_begin = begin;
  //identify the strict minimum. This allows for better numerical accuracy
  double strict_min = (*tmp_begin).softmin;
  int minIdx = 0;
  int i = 0;
  for(; tmp_begin != end; ++tmp_begin) {
  	if(((*tmp_begin).softmin) < strict_min){
  		strict_min = (*tmp_begin).softmin;
  		minIdx = i;
  	}
  	i++;
  }
  assert(begin != end);
  //now we calculate the soft minimum
  double normalization = 0;
  double weight;
  double softmin = 0;
  i = 0;
  tmp_begin = begin;
  for (; tmp_begin != end; ++tmp_begin) {
    if(i == minIdx){
    	normalization += 1;
    	softmin += (*tmp_begin).softmin;
    } else {
  		weight = exp(-((*tmp_begin).softmin - strict_min ) * beta);
  		normalization += weight;
  		softmin += weight * (*tmp_begin).softmin;
  	}
  	i++;
  }
  softmin /= normalization;
  n.softmin = softmin;
  double softmin_dash;
  double tmp_val;
  //now (finally) we calculate the soft derivative.
  for(; begin != end; ++begin){
  	if(i == minIdx){
  		softmin_dash = 1. / normalization;
  	} else{
  		weight = exp(-((*begin).softmin - strict_min ) * beta);
  		weight /= normalization;
  		softmin_dash = (1 - beta * ((*begin).softmin - softmin)) * weight;
  	}
  	for(unsigned int i = 0; i < (*begin).deriv.size() ; i++){
  	  	for(unsigned int j = 0; j < (*begin).deriv.size() ; j++){
  	  		tmp_val = (n.deriv).get(i,j);
  	  		tmp_val += ((*begin).deriv).get(i,j) * softmin_dash;
  	  		n.deriv.set(i,j,tmp_val);
 	 	}
  	}
  }
  return n;
}

template <typename Iterator>
inline
soft_derivative softmin_deriv(std::pair<Iterator, Iterator> &p)
{
  return softmin_deriv(p.first, p.second);
}

#endif
