/* 
 * TCS Alignment Toolbox
 * 
 * Copyright (C) 2013, 2014
 * Benjamin Paaßen, Charlie Krüger, Georg Zentgraf
 * AG Theoretical Computer Science
 * Centre of Excellence Cognitive Interaction Technology (CITEC)
 * University of Bielefeld
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.unibi.citec.tcs.alignment.comparators;

import de.unibi.citec.tcs.alignment.sequence.Value;

/**
 * This specifies the comparator interface slightly to include the case that
 * entities might be deleted/inserted.
 *
 * The replacement operation should use the "compare" function.
 *
 * @author Benjamin Paassen - bpaassen@techfak.uni-bielefeld.de
 * @param <X> The class that can be compared using this comparator.
 */
public interface GapComparator<X extends Value> extends Comparator<X> {

    /**
     * This should be called during an alignment to retrieve the costs for a
     * deletion of value a from the first sequence.
     *
     * Please note that this has to be well-defined even if the input is null.
     *
     * @param a a value.
     *
     * @return a value between 0 and 1.
     */
    public double delete(X a);

    /**
     * This should be called during an alignment to retrieve the costs for an
     * insertion of value b into the first sequence.
     *
     * Please note that this has to be well-defined even if the input is null.
     *
     * @param b a value.
     *
     * @return a value between 0 and 1.
     */
    public double insert(X b);
}
