/* 
 * TCS Alignment Toolbox
 * 
 * Copyright (C) 2013, 2014
 * Benjamin Paaßen, Charlie Krüger, Georg Zentgraf
 * AG Theoretical Computer Science
 * Centre of Excellence Cognitive Interaction Technology (CITEC)
 * University of Bielefeld
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.unibi.citec.tcs.alignment.sequence;

/**
 * This specifies symbolic keywords by their alphabet.
 *
 * @author Benjamin Paassen - bpaassen@techfak.uni-bielefeld.de
 */
public class SymbolicKeywordSpecification extends KeywordSpecification {

    private final Alphabet alphabet;

    public SymbolicKeywordSpecification(Alphabet alphabet, String keyword) {
        super(keyword, ValueType.SYMBOLIC);
        this.alphabet = alphabet;
    }

    /**
     * Returns the Alphabet that defines which symbols are allowed for this
     * keyword.
     *
     * @return the Alphabet that defines which symbols are allowed for this
     * keyword.
     */
    public Alphabet getAlphabet() {
        return alphabet;
    }

    /**
     * {@inheritDoc }
     */
    @Override
    public boolean validate(Value value) {
        if (!super.validate(value)) {
            return false;
        }
        return ((SymbolicValue) value).getAlphabet().equals(alphabet);
    }

    /**
     * {@inheritDoc }
     */
    @Override
    public int hashCode() {
        int hash = 5;
        hash = 97 * hash + super.hashCode();
        hash = 97 * hash + (this.alphabet != null ? this.alphabet.hashCode() : 0);
        return hash;
    }

    /**
     * {@inheritDoc }
     */
    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final SymbolicKeywordSpecification other = (SymbolicKeywordSpecification) obj;
        if (!super.equals(obj)) {
            return false;
        }
        if (this.alphabet != other.alphabet
                && (this.alphabet == null || !this.alphabet.equals(other.alphabet))) {
            return false;
        }
        return true;
    }
}
