/* 
 * TCS Alignment Toolbox
 * 
 * Copyright (C) 2013, 2014
 * Benjamin Paaßen, Charlie Krüger, Georg Zentgraf
 * AG Theoretical Computer Science
 * Centre of Excellence Cognitive Interaction Technology (CITEC)
 * University of Bielefeld
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.unibi.citec.tcs.alignment.sequence;

/**
 * This defines all possible types of values.
 *
 * @author Benjamin Paassen - bpaassen@techfak.uni-bielefeld.de
 */
public enum ValueType {

    /**
     * Symbols that stem from a discrete Alphabet.
     */
    SYMBOLIC,
    /**
     * Numeric vectors (or scalars as a special case).
     */
    VECTOR,
    /**
     * Arbitrary strings.
     */
    STRING;

    /**
     * The corresponding class of this ValueType.
     *
     * @return The corresponding class of this ValueType.
     */
    public Class<? extends Value> getValueClass() {
        switch (this) {
            case SYMBOLIC:
                return SymbolicValue.class;
            case VECTOR:
                return VectorialValue.class;
            case STRING:
                return StringValue.class;
            default:
                //this does not happen!
                return null;
        }
    }

    /**
     * Returns the corresponding specification class for this ValueType.
     *
     * @return the corresponding specification class for this ValueType.
     */
    public Class<? extends KeywordSpecification> getSpecificationClass() {
        switch (this) {
            case SYMBOLIC:
                return SymbolicKeywordSpecification.class;
            case VECTOR:
                return VectorialKeywordSpecification.class;
            case STRING:
                return StringKeywordSpecification.class;
            default:
                //this does not happen!
                return null;
        }
    }
}
