/* 
 * TCS Alignment Toolbox
 * 
 * Copyright (C) 2013, 2014
 * Benjamin Paaßen, Charlie Krüger, Georg Zentgraf
 * AG Theoretical Computer Science
 * Centre of Excellence Cognitive Interaction Technology (CITEC)
 * University of Bielefeld
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.unibi.citec.tcs.alignment.sequence;

/**
 * This specifies vectors by their length.
 *
 * @author Benjamin Paassen - bpaassen@techfak.uni-bielefeld.de
 */
public class VectorialKeywordSpecification extends KeywordSpecification {

    private final int length;

    public VectorialKeywordSpecification(int length, String keyword) {
        super(keyword, ValueType.VECTOR);
        this.length = length;
    }

    /**
     * The length all vectors for this keyword have to have.
     *
     * @return The length all vectors for this keyword have to have.
     */
    public int getLength() {
        return length;
    }

    /**
     * {@inheritDoc }
     */
    @Override
    public boolean validate(Value value) {
        if (!super.validate(value)) {
            return false;
        }
        return ((VectorialValue) value).getVector().length == length;
    }

    /**
     * {@inheritDoc }
     */
    @Override
    public int hashCode() {
        int hash = 7;
        hash = 59 * hash + super.hashCode();
        hash = 59 * hash + this.length;
        return hash;
    }

    /**
     * {@inheritDoc }
     */
    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final VectorialKeywordSpecification other = (VectorialKeywordSpecification) obj;
        if (!super.equals(obj)) {
            return false;
        }
        if (this.length != other.length) {
            return false;
        }
        return true;
    }
}
