/* 
 * TCS Alignment Toolbox
 * 
 * Copyright (C) 2013, 2014
 * Benjamin Paaßen, Charlie Krüger, Georg Zentgraf
 * AG Theoretical Computer Science
 * Centre of Excellence Cognitive Interaction Technology (CITEC)
 * University of Bielefeld
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.unibi.citec.tcs.alignment;

import de.unibi.citec.tcs.alignment.comparators.ReplacementComparator;
import de.unibi.citec.tcs.alignment.sequence.Sequence;
import junit.framework.TestCase;

/**
 *
 * @author Benjamin Paassen - bpaassen@techfak.uni-bielefeld.de
 */
public class SoftAlignmentScoreAlgorithmTest extends TestCase {

    public SoftAlignmentScoreAlgorithmTest(String testName) {
        super(testName);
    }

    @Override
    protected void setUp() throws Exception {
        super.setUp();
    }

    @Override
    protected void tearDown() throws Exception {
        super.tearDown();
    }

    /**
     * Test of calculateSoftAlignment method, of class SoftAlignmentAlgorithm.
     */
    public void testPerformance() {

        //align a...a and a...b
        final int length = 1000;

        final StringBuilder aBuilder = new StringBuilder();
        final StringBuilder bBuilder = new StringBuilder();

        aBuilder.append('b');
        bBuilder.append('b');
        for (int i = 1; i < length - 1; i++) {
            aBuilder.append('a');
            bBuilder.append('a');
        }
        aBuilder.append('a');
        bBuilder.append('b');

        final Sequence a = new Sequence(aBuilder.toString());
        final Sequence b = new Sequence(bBuilder.toString());
        //initialize actual instance.
        final SoftAlignmentScoreAlgorithm instance
            = new SoftAlignmentScoreAlgorithm(new AlignmentSpecification(a));
        instance.setBeta(10);

        final double result = instance.calculateAlignment(a, b);
        //the result should be very close to zero because we normalize using the length.
        assertEquals(0, result, 0.01);
    }

    /**
     * Test of calculateSoftAlignment method, of class SoftAlignmentAlgorithm.
     */
    public void testCalculateSoftAlignment() {
        final Sequence a = new Sequence("ab");
        final Sequence b = new Sequence("aab");
        final SoftAlignmentScoreAlgorithm instance
            = new SoftAlignmentScoreAlgorithm(new AlignmentSpecification(a));
        double optimum = ((ReplacementComparator) instance.getSpecification().getComparator(0)).getCostMatrix().getInsertionCost("b") / 5;

        //check if the result is bigger than the optimum but converges against
        //the optimum for higher beta.
        double lastResult = 5;
        for (double beta = 0; beta < 20; beta++) {
            instance.setBeta(beta);
            final double result = instance.calculateAlignment(a, b);
            assertTrue(result <= lastResult);
            lastResult = result;
        }
        assertEquals(optimum, lastResult, 0.0001);
    }
}
