/* 
 * TCS Alignment Toolbox
 * 
 * Copyright (C) 2013, 2014
 * Benjamin Paaßen, Charlie Krüger, Georg Zentgraf
 * AG Theoretical Computer Science
 * Centre of Excellence Cognitive Interaction Technology (CITEC)
 * University of Bielefeld
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.unibi.citec.tcs.alignment;

import junit.framework.TestCase;

/**
 *
 * @author Benjamin Paassen - bpaassen@techfak.uni-bielefeld.de
 */
public class SparseMatrixTest extends TestCase {

    public SparseMatrixTest(String testName) {
        super(testName);
    }

    @Override
    protected void setUp() throws Exception {
        super.setUp();
    }

    @Override
    protected void tearDown() throws Exception {
        super.tearDown();
    }

    /**
     * Test of get method, of class SparseMatrix.
     */
    public void testGet() {
        final SparseMatrix<String> instance = new SparseMatrix<String>();
        assertNull(instance.get(0, 0));
        assertNull(instance.get(1, 0));
        assertNull(instance.get(20, 0));
        instance.set(20, 0, "test");
        assertEquals("test", instance.get(20, 0));
        assertNull(instance.get(0, 0));
        assertNull(instance.get(1, 0));
    }

    /**
     * The SparseMatrix should scale linearly with size, not quadratically.
     */
    public void testPerformance() {
        for (int n = 10; n <= 1000; n *= 10) {
            //use double matrix for comparison.
            double start = System.nanoTime();
            final double[][] matrix = new double[n][n];
            for (int i = 0; i < n; i++) {
                matrix[i][i] = 4;
            }
            double time = (System.nanoTime() - start) / 1E6;
            System.out.println("Baseline (double[][]) = " + time + " ms");
            start = System.nanoTime();
            final SparseMatrix<Double> sparseMatrix = new SparseMatrix<Double>();
            for (int i = 0; i < n; i++) {
                sparseMatrix.set(n, n, 4.);
            }
            time = (System.nanoTime() - start) / 1E6;
            System.out.println("sparse = " + time + " ms");
        }
    }

    /**
     * Test of toString method, of class SparseMatrix.
     */
    public void testToString() {
        final SparseMatrix<String> instance = new SparseMatrix<String>();
        instance.set(20, 0, "test");
        instance.set(1, 11, "test");
        //make the string representation unique.
        System.out.println(instance.toString());
    }

}
