/*
 * TCS Alignment Toolbox
 * 
 * Copyright (C) 2013, 2014
 * Benjamin Paaßen, Charlie Krüger, Georg Zentgraf
 * AG Theoretical Computer Science
 * Centre of Excellence Cognitive Interaction Technology (CITEC)
 * University of Bielefeld
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.unibi.citec.tcs.alignment;

import de.unibi.citec.tcs.alignment.sequence.Alphabet;
import de.unibi.citec.tcs.alignment.sequence.Sequence;
import java.util.EnumMap;
import static junit.framework.Assert.assertEquals;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import static org.junit.Assert.*;

/**
 *
 * @author Benjamin Paassen - bpaassen@techfak.uni-bielefeld.de
 */
public class StrictLocalAlignmentScoreAlgorithmTest {

	public StrictLocalAlignmentScoreAlgorithmTest() {
	}

	@BeforeClass
	public static void setUpClass() {
	}

	@AfterClass
	public static void tearDownClass() {
	}

	@Before
	public void setUp() {
	}

	@After
	public void tearDown() {
	}

	/**
	 * Test of calculateAlignment method, of class StrictAlignmentAlgorithm.
	 */
	@Test
	public void testAlignmentPerformance() {

		//align ba...a and ba...b
		final int length = 1000;

		final StringBuilder aBuilder = new StringBuilder();
		final StringBuilder bBuilder = new StringBuilder();

		aBuilder.append('b');
		bBuilder.append('b');
		for (int i = 1; i < length - 1; i++) {
			aBuilder.append('a');
			bBuilder.append('a');
		}
		aBuilder.append('a');
		bBuilder.append('b');

		final Alphabet alpha = new Alphabet("a|b");

		final Sequence a = new Sequence(aBuilder.toString(), alpha);
		final Sequence b = new Sequence(bBuilder.toString(), alpha);

		//initialize actual instance.
		final StrictLocalAlignmentScoreAlgorithm instance
				= new StrictLocalAlignmentScoreAlgorithm(
						new AlignmentSpecification(a));

		final long start = System.nanoTime();
		final double result = instance.calculateAlignment(a, b);
		final double time = (double) (System.nanoTime() - start) / 1E9;
		assertEquals(0, result, 0.01);
	}

}
