/* 
 * TCS Alignment Toolbox
 * 
 * Copyright (C) 2013, 2014
 * Benjamin Paaßen, Charlie Krüger, Georg Zentgraf
 * AG Theoretical Computer Science
 * Centre of Excellence Cognitive Interaction Technology (CITEC)
 * University of Bielefeld
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.unibi.citec.tcs.alignment.comparators;

import de.unibi.citec.tcs.alignment.OperationType;
import de.unibi.citec.tcs.alignment.comparators.SparseLocalDerivative.SparseDeriativeEntry;
import de.unibi.citec.tcs.alignment.sequence.Alphabet;
import de.unibi.citec.tcs.alignment.sequence.SymbolicValue;
import java.util.Iterator;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import static org.junit.Assert.*;

/**
 *
 * @author Benjamin Paassen - bpaassen@techfak.uni-bielefeld.de
 */
public class SparseDerivableComparatorTest {

	public SparseDerivableComparatorTest() {
	}

	@BeforeClass
	public static void setUpClass() {
	}

	@AfterClass
	public static void tearDownClass() {
	}

	@Before
	public void setUp() {
	}

	@After
	public void tearDown() {
	}

	/**
	 * Test of calculateSparseLocalDerivative method, of class
	 * SparseDerivableComparator.
	 */
	@Test
	public void testCalculateSparseLocalDerivative() {
		final Alphabet alpha = new Alphabet("a|b");
		//check TrivialEditComparator
		final TrivialEditComparator trivial = new TrivialEditComparator();
		SparseLocalDerivative actual = trivial.calculateSparseLocalDerivative(
			new SymbolicValue(alpha, "a"), null, OperationType.DELETION);
		checkSparseDerivative(0, actual);
		actual = trivial.calculateSparseLocalDerivative(
			new SymbolicValue(alpha, "a"), null, OperationType.SKIPDELETION);
		checkSparseDerivative(2, actual);
		actual = trivial.calculateSparseLocalDerivative(
			null, new SymbolicValue(alpha, "a"), OperationType.INSERTION);
		checkSparseDerivative(1, actual);
		actual = trivial.calculateSparseLocalDerivative(
			null, new SymbolicValue(alpha, "a"), OperationType.SKIPINSERTION);
		checkSparseDerivative(3, actual);
		actual = trivial.calculateSparseLocalDerivative(
			new SymbolicValue(alpha, "a"), new SymbolicValue(alpha, "a"),
			OperationType.REPLACEMENT);
		checkSparseDerivative(4, actual);
		actual = trivial.calculateSparseLocalDerivative(
			new SymbolicValue(alpha, "a"), new SymbolicValue(alpha, "b"),
			OperationType.REPLACEMENT);
		checkSparseDerivative(5, actual);

		//check ReplacementComparator
		final ReplacementComparator rep = new ReplacementComparator(
			new ReplacementCosts(alpha, true, true));
		actual = rep.calculateSparseLocalDerivative(
			new SymbolicValue(alpha, "a"), null, OperationType.DELETION);
		checkSparseDerivative(0 * 4 + 2, actual);
		actual = rep.calculateSparseLocalDerivative(
			new SymbolicValue(alpha, "b"), null, OperationType.DELETION);
		checkSparseDerivative(1 * 4 + 2, actual);
		actual = rep.calculateSparseLocalDerivative(
			new SymbolicValue(alpha, "a"), null, OperationType.SKIPDELETION);
		checkSparseDerivative(0 + 3, actual);
		actual = rep.calculateSparseLocalDerivative(
			null, new SymbolicValue(alpha, "a"), OperationType.INSERTION);
		checkSparseDerivative(2 * 4 + 0, actual);
		actual = rep.calculateSparseLocalDerivative(
			null, new SymbolicValue(alpha, "b"), OperationType.INSERTION);
		checkSparseDerivative(2 * 4 + 1, actual);
		actual = rep.calculateSparseLocalDerivative(
			null, new SymbolicValue(alpha, "a"), OperationType.SKIPINSERTION);
		checkSparseDerivative(3 * 4 + 0, actual);
		actual = rep.calculateSparseLocalDerivative(
			new SymbolicValue(alpha, "a"), new SymbolicValue(alpha, "a"),
			OperationType.REPLACEMENT);
		checkSparseDerivative(0 + 0, actual);
		actual = rep.calculateSparseLocalDerivative(
			new SymbolicValue(alpha, "a"), new SymbolicValue(alpha, "b"),
			OperationType.REPLACEMENT);
		checkSparseDerivative(0 + 1, actual);

	}

	private static void checkSparseDerivative(int expectedIndex, SparseLocalDerivative actual) {
		final Iterator<SparseDeriativeEntry> iterator = actual.iterator();
		assertTrue(iterator.hasNext());
		final SparseDeriativeEntry entry = iterator.next();
		assertFalse(iterator.hasNext());
		assertEquals(entry.getKey(), new Integer(entry.getParameterIndex()));
		assertEquals(entry.getValue(), new Double(entry.getDerivative()));
		assertEquals(expectedIndex, entry.getParameterIndex());
		assertEquals(1, entry.getDerivative(), 0);
	}

}
