/*
 * TCS Alignment Toolbox
 * 
 * Copyright (C) 2013, 2014
 * Benjamin Paaßen, Charlie Krüger, Georg Zentgraf
 * AG Theoretical Computer Science
 * Centre of Excellence Cognitive Interaction Technology (CITEC)
 * University of Bielefeld
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.unibi.citec.tcs.alignment.wrappers;

import de.unibi.citec.tcs.alignment.sequence.Alphabet;
import de.unibi.citec.tcs.alignment.sequence.KeywordSpecification;
import de.unibi.citec.tcs.alignment.sequence.Node;
import de.unibi.citec.tcs.alignment.sequence.NodeSpecification;
import de.unibi.citec.tcs.alignment.sequence.Sequence;
import de.unibi.citec.tcs.alignment.sequence.StringKeywordSpecification;
import de.unibi.citec.tcs.alignment.sequence.SymbolicKeywordSpecification;
import de.unibi.citec.tcs.alignment.sequence.VectorialKeywordSpecification;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import static org.junit.Assert.*;

/**
 *
 * @author Benjamin Paassen - bpaassen@techfak.uni-bielefeld.de
 */
public class RandomSequenceGeneratorTest {

	public RandomSequenceGeneratorTest() {
	}

	@BeforeClass
	public static void setUpClass() {
	}

	@AfterClass
	public static void tearDownClass() {
	}

	@Before
	public void setUp() {
	}

	@After
	public void tearDown() {
	}

	/**
	 * Test of generateSequences method, of class RandomSequenceGenerator.
	 */
	@Test
	public void testGenerateSequences_3args_1() {
		final int N = 5;
		final int L = 10;
		final int K = 6;
		final Sequence[] result
				= RandomSequenceGenerator.generateSequences(N, L, K);
		assertEquals(N, result.length);
		for (int n = 0; n < N; n++) {
			assertNotNull(result[n]);
			assertEquals(L, result[n].getNodes().size());
			assertEquals(K, result[n].getNodeSpecificiation().size());
			for (int l = 0; l < L; l++) {
				final Node node = result[n].getNodes().get(l);
				assertNotNull(node);
				for (int k = 0; k < K; k++) {
					assertNotNull(node.getValue(k));
				}
			}
		}
	}

	/**
	 * Test of generateSequences method, of class RandomSequenceGenerator.
	 */
	@Test
	public void testGenerateSequences_3args_2() {
		final int N = 5;
		final int L = 10;
		final Alphabet alpha = new Alphabet("a|b");
		final KeywordSpecification[] keywordSpecs = {
			new SymbolicKeywordSpecification(alpha, "one"),
			new VectorialKeywordSpecification(5, "two"),
			new StringKeywordSpecification("three")
		};
		final NodeSpecification expected = new NodeSpecification(keywordSpecs);
		final Sequence[] result
				= RandomSequenceGenerator.generateSequences(N, L, expected);
		assertEquals(N, result.length);
		for (int n = 0; n < N; n++) {
			assertNotNull(result[n]);
			assertEquals(L, result[n].getNodes().size());
			assertEquals(expected, result[n].getNodeSpecificiation());
			for (int l = 0; l < L; l++) {
				final Node node = result[n].getNodes().get(l);
				assertNotNull(node);
				for (int k = 0; k < keywordSpecs.length; k++) {
					assertNotNull(node.getValue(k));
				}
			}
		}
	}

}
